{% extends "!page.html" %}
{%- block scripts %}
    {{ super() }}
    <link rel="stylesheet" type="text/css" href="{{ pathto('_static/sphinx_asdf.css', 1) }}">
    <script type="text/javascript" src="{{ pathto('_static/sphinx_asdf.js', 1) }}"></script>
{%- endblock %}
{% block extrahead %}
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/3.4.1/css/bootstrap.min.css" integrity="sha384-HSMxcRTRxnN+Bdg0JdbxYKrThecOKuH5zCYotlSAcp1+c8xmyTe9GYg1l9a69psu" crossorigin="anonymous">
    <script src="https://stackpath.bootstrapcdn.com/bootstrap/3.4.1/js/bootstrap.min.js" integrity="sha384-aJ21OjlMXNL5UyIl/XNwTMqvzeRMZH2w8c5cRVpzpU8Y5bApTppSuUkhZXN0VxHd" crossorigin="anonymous"></script>
{% endblock %}
{% block footer %}
    {{ super() }}
    <div class="container">
        Schema documentation automatically generated by <a href="http://github.com/spacetelescope/sphinx-asdf">sphinx-asdf</a> {{ sphinx_asdf_version }}.
    </div>
{% endblock %}
